<?php
/*
Plugin Name: Contact SiteDart Support
Plugin URI:
Description: Contact SiteDart support with just a click of a button. This plugin adds a SiteDart support button to your admin panel.
Version: 0.2.1
Author: SiteDart Studio
Author URI: http://sitedartstudio.com
License: Proprietary
*/

if (!defined('PROCESSING')) {
	define('PROCESSING',TRUE);
}

class contact_sitedart_support
{
	/**
	 * Internal version number.
	 * @var string
	 */
	protected static $version = "0.2.1";

	protected $config;
	protected static $config_option;
	//protected $user_is_admin;
	protected static $dir;
	protected static $url;

	public function __construct()
	{
		global $current_user;
		//get_currentuserinfo();

		self::$dir = plugin_dir_path( __FILE__ );
		self::$url = plugin_dir_url( __FILE__ );

		self::$config_option = 'sitedart_contact_support_config';

		// plugin activation
		register_activation_hook( __FILE__, array($this, 'activate') );

		// plugin deactivation
		register_deactivation_hook( __FILE__, array($this, 'deactivate') );

		//$this->user_is_admin = FALSE;

		//attempt to retrieve config options
		$this->config = get_option(self::$config_option);

		//determine if plugin is active
		if (empty($this->config)) {
			//get domain
			$domain = parse_url(site_url(),PHP_URL_HOST);
			$domain = str_replace('www.', '', $domain);
			//set config
			$this->config = array(
				'active' => 0,
				'api_key' => '',
				'registered_domain' => $domain,
			);
			return;
		}
		if (!isset($this->config['active']) || empty($this->config['active'])) return;

		//initialize everything need for form(s)

		//we are active; run init;
		add_action( 'init', array( $this, 'init' ) );

		// only run admin stuff if the are on admin pages
		if ( is_admin() ) {
			//add_action( "admin_init", array( $this, "admin_init" ) );
			add_action('admin_menu', array($this, 'admin_menu_config'));
			$plugin = plugin_basename(__FILE__);
			add_filter( "plugin_action_links_$plugin", array( $this, "plugin_action_links" ), 10, 4 );
		}
	}


	public function init()
	{
		//only run admin init if user is has permissions
		if (!current_user_can('manage_options')) return FALSE;

		//button in top admin bar
		add_action( 'admin_bar_menu', array($this, 'support_button'), 999 );
		add_action( 'wp_dashboard_setup', array($this,'add_dashboard_widgets') );

	}

	/**
	 * initialize everything needed for admin
	 */
	public function admin_init()
	{
		//only run admin init if user is has permissions
		if (!current_user_can('manage_options')) return FALSE;
	}


	public function activate()
	{
		//set defaults and add options
		$defaults_config = $this->config;

		$defaults_config['active'] = 1;

		update_option(self::$config_option, $defaults_config);
	}


	public function deactivate()
	{
		//disable plugin functionality
		$config =  $this->config ;

		$config['active'] = 0;

		update_option(self::$config_option, $config);

		//remove admin config menu item
		remove_action('admin_menu', array($this, 'menu_config'));

		//remove button in top admin bar
		remove_action( 'admin_bar_menu', array($this,'support_button') );

		//remove dashboard widget
		add_action( 'wp_dashboard_setup', array($this,'add_dashboard_widgets') );
	}


	public static function uninstall()
	{
		//remove plugin and options
		delete_option(self::$config_option);
	}


	/**
	 * return a form view
	 *
	 * @author		John Hocker <jhocker@dnet.net>
	 * @param 		array [$data]
	 * @return		string html form
	 * @since		2014-03-27
	 */
	public function get_settings_form($data)
	{
		$config_form = '
			<form class="form-horizontal" method="post">
			<fieldset>

			<!-- Form Name -->
			<legend>Contact Support Settings</legend>

			<!-- Text input-->
			<div class="control-group">
			  <label class="control-label" for="sd_contact_api_key">API key</label>
			  <div class="controls">
				<input id="sd_contact_api_key" name="api_key" type="text" placeholder="API key" class="input-xlarge" value="{api_key}"/>
				<p class="help-block">Enter your API key</p>
			  </div>
			</div>

			<!-- Button -->
			<div class="control-group">
			  <label class="control-label" for="submit"></label>
			  <div class="controls">
				<button id="submit" name="submit" class="btn btn-success">Submit</button>
			  </div>
			</div>

			</fieldset>
			</form>
		';

		return $this->parse($config_form,$data);
	}


	/**
	 * Displays the contact form settings page
	 *
	 * @author		John Hocker <jhocker@dnet.net>
	 * @param 		array [$atts] shortcode atributes
	 * @return		boolean
	 * @since		2014-03-27
	 */
	public function settings_form()
	{
		// if user not logged in. do nothing
		if ( !current_user_can('manage_options') ) return FALSE;

		$input = $this->post();

		$page_content = "";
		$do_update = FALSE;

		$defaults = $this->config;

		if (!empty($input)) {
			foreach ($defaults as $key => $value) {
				if ($key == 'active') continue;

				if ( isset($input[$key]) /*&& !empty($input[$key])*/ ) {
					$defaults[$key] = $input[$key];
					$do_update = TRUE;
				}
			}
		}

		if ( $do_update )
		{
			if(update_option(self::$config_option, $defaults)){
				$page_content = '<div class="updated">Settings saved.</div>';
			}
		}

		$page_content .= $this->get_settings_form($defaults);

		echo $page_content;
	}


	protected function parse($view,$data)
	{
		$l_delim = '{';
		$r_delim = '}';

		if (empty($data)) {
			return $view;
		}

		foreach ($data as $key => $value) {
			$view = str_replace($l_delim.$key.$r_delim, $value, $view);
		}

		return $view;
	}


	protected function post()
	{
		$s = $_POST;

		/*
		$s = json_encode($s, JSON_HEX_APOS);

		$search[] = '\\r';
		$replace[] = '\\\\\\r';
		$search[] = '\\n';
		$replace[] = '\\\\\\n';

		$s = str_replace($search,$replace,$s);

		$s = json_decode(stripslashes($s), TRUE);
		 */

		return $s;
	}


	public function support_button($wp_admin_bar)
	{
		$args = array(
		'id' => 'sitedart-issue-btn',
		'title' => 'Get Support',
		'href' => '#',
		'meta' => array(
			'html' => $this->support_form(),
			'title' => 'Submit a support ticket',
			'onclick' => 'document.getElementById("sitedart-support-form").submit();return false;',
			)
		);
		$wp_admin_bar->add_node( $args );
	}

	public function menu_item_submit_issue()
	{
		$id = 'sd-support-autoload-form';
		//$form = $this->support_form($id);
		$page_content = '<p>Having problems? Click the "Get Support" button to contact SiteDart Support.</p>';
		$page_content .= $this->support_form($id,TRUE);
		$page_content .= '<script type="text/javascript">
				function sd_get_support(){
					document.getElementById("'.$id.'").submit();
				}
				window.onload = sd_get_support();
			</script>';

		echo $page_content;
	}


	public function admin_menu_config()
	{
		add_menu_page('SiteDart Support Settings', 'SiteDart Support', 'manage_options','contact-sitedart-support-option');
		add_submenu_page('contact-sitedart-support-option', 'SiteDart Support Issue', 'Submit Issue', 'manage_options','contact-sitedart-support-option', array($this, 'menu_item_submit_issue'));
		add_submenu_page('contact-sitedart-support-option', 'SiteDart Support Settings', 'Settings', 'manage_options','contact-sitedart-support-setting', array($this, 'settings_form'));
	}

	protected function support_form($id = NULL, $with_btn = FALSE)
	{
		// only get site url if geting from admin page. else get permalink to current page
		$url = ( is_admin() )? site_url() : get_permalink();
		//$url = $this->config['registered_domain'];

		$data = array(
			'client_id' => $this->config['api_key'],
			'domain' => $this->config['registered_domain'],
			'website' => $this->config['registered_domain'],
			'referer' => $url,
		);

		$btn = ( empty($with_btn) )?'':'<input type="submit" value="Get Support" id="sd_support_form_submit">';
		$id = ( empty($id) )?"sitedart-support-form":$id;

		$html = '<form id="'.$id.'" action="http://sitedartstudio.com/contact/support" method="post" target="_blank">
					<input type="hidden" name="client_id" value="{client_id}"/>
					<input type="hidden" name="website" value="{website}"/>
					<input type="hidden" name="referer" value="{referer}"/>
					<input type="hidden" name="reg_domain" value="{domain}"/>'.$btn.'
				</form>';

		return $this->parse($html, $data);
	}


	public function plugin_action_links( $actions, $file, $data, $context ) {
		$settings_link = esc_url( add_query_arg( array( 'page' => 'contact-sitedart-support-setting' ), admin_url( 'admin.php' ) ) );
		$actions[ 'settings' ] = '<a href="' . $settings_link . '">Settings</a>';

		return $actions;
	}

	public function add_dashboard_widgets()
	{
		wp_add_dashboard_widget(
			'contact_sitedart_support_dashboard_widget',// Widget slug.
			'SiteDart Support',						// Title.
			array($this,'widget_support_form') // Display function.
		);
	}


	public function widget_support_form()
	{
		$id = 'sd-support-widget-form';
		$page_content = '<p>Having problems? Click the "Get Support" button to contact SiteDart Support.</p>';
		$page_content .= $this->support_form($id,TRUE);

		echo $page_content;
	}

}

new contact_sitedart_support;

// plugin uninstall
register_uninstall_hook( __FILE__, array('contact_sitedart_support', 'uninstall') );

?>